% reprogram the Maple file for calculating 1D Weiss osc in Graphene
% this will be expanded for 2D Weiss osc

clear
% physical constants
hbar = 1.055e-34; 
e = 1.6022e-19; 
h = 2*pi*hbar; 
kB = 1.3806e-23;

% sample specific, constant:
lfree = 1e-6; % mean free path
V0 = 0.01*e; % modulation strength in Joule
% V0 does not enter into Phi, it only enters in A0, and then in the
% quantitative value of sigma. But this will not be plotted
a0 = 39e-9; % lattice period
K = 2*pi/a0;
T = 125; % Kelvin 
beta = 1/(kB*T);
B0 = hbar/(e*a0^2);  % must still define b = B/B0
vF = 1e6;  % Graphene Fermi velocity
tau = lfree/vF;
A0 = (8*pi^2*e^2/h)*(V0^   2*tau*beta/hbar);

% Simulation constants
n_max = 30; % Highest LL to be considered: Speeds up things at low B and high density
LL_range = 10; % range in units of kB T around EF to include 
max_denominator = 20; % max denominator for Hofstadter routine
gap_removal = 0.2; % this is scaled with the bandwidth of the LL under consideration
% good values: 30, 10 kBT, max denom 20, gap removal 0.2



%nS_vec = 0.1e16:0.1e16:3e16; 
%nS_vec = -3e16:0.05e16:3e16; 
nS_vec = -5e16:0.04e16:5e16; % values for publication graph
%nS_vec = -5e16:0.2e16:5e16; 

B_vec = 0.1:0.02:10;


Phi_mat = double.empty;
Phi_2D_mat = double.empty;
bw_mat = double.empty;
nF_mat = double.empty;
test_mat = double.empty;

for nS = nS_vec

nS    % display some progress
    
kF = sqrt(abs(pi*nS));  % take abs of nS
EF = hbar*vF*kF;  % then, EF will be positive. Is that a good idea?
p = kF*a0;

Phi=double.empty;
Phi_2D=double.empty;
bw = double.empty;
nF = double.empty;
test = double.empty;

for B=B_vec
b = B/B0;

l = sqrt(hbar/(e*B)); % magnetic length at given field
u = K^2*l^2/2;
sum = 0;
sum_2D = 0;


% first we have to extract the Hofstadter lines
alpha = 1/(B*a0^2/(h/e)); % inverse flux per unit cell
intervals = HofstadterLine(alpha,max_denominator); 

% for plotting Landau level index, let's store the LL at EF in an array for
% later

nF(end + 1) = round(EF^2/(2*vF^2*hbar*e*B));  %  store LL index at Fermi level for future use


% only consider LLs within a given range (in units of kB T) around EF
% LL must be at least one, and cut off to n_max 

Elim = EF+LL_range*kB*T;
n2 = min(n_max, round(Elim^2/(2*vF^2*hbar*e*B))+1);
Elim = EF-LL_range*kB*T;
n1 = max(1, round(Elim^2/(2*vF^2*hbar*e*B))-1);

bw(end + 1)=0;
test(end + 1)=0;  

for n = n1:n2
    En = (vF*hbar/l)*sqrt(2*n);
    Lsum = Laguerre(n,u) + Laguerre(n-1,u); % to save a little time
    bandwidth = abs(exp(-u/2)*Lsum);
    if n==nF(end)  % we hit the current LL, let's see if we get all of them
        bw(end)=bandwidth;  % once more, we create some arrays for diagnostic plotting later on
    end
    
    
    min_gap = min([8 gap_removal/bandwidth]);  % calculate minimum surviving gap 
    % since the Hofstadter line has to be scaled with the Landau band width
    
    %min_gap = 1; % by way of testing....
    
    broadened = Broaden(intervals,min_gap);  
    % now, too small gaps in the Hofstadter line have been kicked out
    bstart = broadened(:,1);
    bend = broadened(:,2);
    len = bend - bstart; % array of allowed widths of energy bands (len is a vector!)
    cond = exp(beta*(En-EF))/(exp(beta*(En-EF))+1)^2*(Lsum)^2;
    sum = sum + cond;
    if n==nF(end)  % we hit the current LL, let's see if we get all of them
        test(end)=transpose(len)*len/64;  % once more, we create some arrays for diagnostic plotting later on
    end
    cond = cond*(transpose(len)*len/64);  % for 2D reduce those by Hofstadter subbands
    % convenient way of calculating sum over square of all lengths as a
    % scalar product
    
    
 
    sum_2D = sum_2D + cond;
end
sum = sum*u*exp(-u)/4;
sum_2D = sum_2D*u*exp(-u)/4;



Phi(end + 1)=sum; % append current data to current line in matrix
Phi_2D(end + 1)=sum_2D;

end
%hold on
%plot(B_vec,Phi);
%plot(B_vec,Phi_2D);
%hold off
%axis([-inf inf 0 0.1]);
Phi_mat(end+1,1:length(Phi)) = Phi;
Phi_2D_mat(end+1,1:length(Phi_2D)) = Phi_2D;
bw_mat(end+1,1:length(bw)) = bw;
nF_mat(end+1,1:length(nF)) = nF;
test_mat(end+1,1:length(test)) = test;
% append all lines to the result matrix

end
 %imagesc(B_vec,nS_vec,Phi_2D_mat);

 % calcuations are done, now some plotting commands

 
 %%
% Generate Figure 5a of the paper

clf

data = (Phi_mat - min(min(Phi_mat)))/max(max(Phi_mat)); % for COs only


data = (abs(data));

data1 = data;


ax1 = axes('position',[0.12 0.15 0.8 0.78],'Box','on'); 
axes(ax1)

hold on


imagesc(B_vec/(B0*2*pi),nS_vec,data1,[0.0 0.3]);  % for COs

 
 set(gca,'ydir','normal','fontsize',18,'linewidth',1.5,'box','on','layer','top')
 colormap(flip(gray));
 ylabel({'n_S (10^{16} m^{-2})'},'fontsize',20);

 ylim([min(nS_vec)*1.01 max(nS_vec)*1.01]);
xlim([0 3.5]);  

yticks(-4e16:2e16:4e16);
yticklabels([-4 2 0 2 4]);

 
 
% Create xlabel
xlabel({'\phi/\phi_0'},'fontsize',20);
colorbar
for lambda=1:5
nS_fb = (lambda - 0.25)^2*a0^2*e^2*B_vec.^2/(4*pi*hbar^2);
plot(B_vec/(B0*2*pi),nS_fb,'LineWidth',1.5,'Color','red');
plot(B_vec/(B0*2*pi),-nS_fb,'LineWidth',1.5,'Color','red');
end


text(-0.5,5e16,'\bf a','FontSize',36)

hold off
ax1.FontSize = 24;
ax1.LineWidth = 1.5;

 

%%
% Generate Figure 5b of the paper

clf
data = (Phi_2D_mat - min(min(Phi_2D_mat)))/max(max(Phi_2D_mat)); % For BZ CO


data = (abs(data));

data1 = data;

data1(1:fix(size(nS_vec,2)/2),:) = (data1(1:fix(size(nS_vec,2)/2),:) - 0.165)*(0.25/0.035); % increase contrast: Definite version
% increase contrast in lower half of the image

ax1 = axes('position',[0.12 0.15 0.8 0.78],'Box','on'); 
axes(ax1)

hold on

imagesc(B_vec/(B0*2*pi),nS_vec,data1,[0.0 0.25]);  % for BZ + COs  % plot this twice to avoid problems with transparency in upper half
rectangle('Position',[0.02 -4.98e16 3.46 4.98e16],'EdgeColor',[0 0.5 0],'LineStyle','--','LineWidth',1.5,'FaceColor',[0.7 1 0.7])
% Green rectangle to highlight lower portion

im = imagesc(B_vec/(B0*2*pi),nS_vec,data1,[0.0 0.25]);  % for BZ + COs
im.AlphaData=10*data1;  % regardless of alpha, the upper half will come out exactly as desired
% Generate Alpha channel so that peaks in lower half are still visible, but
% background is now green instead of white


 set(gca,'ydir','normal','fontsize',18,'linewidth',1.5,'box','on','layer','top')
 colormap(flip(gray));
 ylabel({'n_S (10^{16} m^{-2})'},'fontsize',20);

 ylim([min(nS_vec)*1.01 max(nS_vec)*1.01]);
xlim([0 3.5]);  % Feb 2021 version

yticks(-4e16:2e16:4e16);
yticklabels([-4 2 0 2 4]);

 
 
% Create xlabel
xlabel({'\phi/\phi_0'},'fontsize',20);
colorbar
for lambda=1:5
nS_fb = (lambda - 0.25)^2*a0^2*e^2*B_vec.^2/(4*pi*hbar^2);
plot(B_vec/(B0*2*pi),nS_fb,'LineWidth',1.5,'Color','red');
plot(B_vec/(B0*2*pi),-nS_fb,'LineWidth',1.5,'Color','red');
end



plot([1/2 1/2],[5e16 1e16],'--','Color','red');
plot([1/3 1/3],[5e16 1e16],'--','Color','red');
plot([1/4 1/4],[5e16 1e16],'--','Color','red');
plot([1/5 1/5],[5e16 1e16],'--','Color','red');
plot([1/6 1/6],[5e16 1e16],'--','Color','red');
% 
plot([2/3 2/3],[5e16 1e16],'--','Color','blue');
plot([2/5 2/5],[5e16 1e16],'--','Color','blue');
% 
plot([2/7 2/7],[5e16 1e16],'--','Color','blue');






rectangle('Position',[0.02 -4.98e16 3.46 4.98e16],'EdgeColor',[0 0.2 0],'LineStyle','--','LineWidth',1.5)
% dashed line to be dark green

text(1.8,-1.5e16,'Enhanced contrast','Color',[0 0.2 0],'FontSize',24 )

text(-0.5,5e16,'\bf b','FontSize',36)

hold off
ax1.FontSize = 24;
ax1.LineWidth = 1.5;


%%

% Generate Supplemental Figure

surf(B_vec/(B0*2*pi),nS_vec/1e16,Phi_2D_mat,'EdgeColor','black','EdgeAlpha',0.4,'FaceColor','interp');

%surf(B_vec/(B0*2*pi),nS_vec,Phi_2D_mat,'EdgeColor','none','FaceColor','red','FaceLighting','gouraud');

colormap(hsv);
view([31 16]);
xlabel({'\phi/\phi_0'});
ylabel({'n_S (10^{16} m^{-2})'});
zlabel('\sigma_{bc,2D} (arb. units)')
ax1=gca;
ax1.FontSize = 24;

%%




%%